
/***************************************************************************
 *                    aKpi - a QT/KDE acpi frontend                        *
 *                                                                         *
 *   file:              acpibase.cpp                                       *
 *   copyright:         (c) 2002 by Tom Winkler                            *
 *   email:             akpi@aon.at                                        *
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "acpibase.h"

#include "acpinode.h"
#include "acpiroot.h"
#include "acpibattery.h"
#include "acpiacadapter.h"
#include "acpiprocessor.h"
#include "acpithermal.h"
#include "acpitoshiba.h"
#include "acpilist.h"

using namespace std;


// ------------------------------------------------------------
// AcpiBase::AcpiBase
// ------------------------------------------------------------
/*!
	The constructor takes a pointer to an AcpiList object, a pointer to a QWidget which is
	the parent of the newly created object within the GUI and a pointer to a QListView which
	is the parent ot the newly created object within the aKpi browser.
	The QHBox hbox is the widget where the node hast to place all of it's GUI elements onto
	(if there are none just leave it empty).
*/
AcpiBase::AcpiBase(AcpiList* list, QWidget* wparent, QListView* lvparent) : QHBox(wparent), QListViewItem(lvparent)
{
	name = "AcpiBase";
	this->list = list;
	this->wparent = wparent;
	pollType = none;
	hbox = static_cast<QHBox*>(this);
	hbox->hide();
}


// ------------------------------------------------------------
// AcpiBase::AcpiBase
// ------------------------------------------------------------
/*!
	The constructor takes a pointer to an AcpiList object, a pointer to a QWidget which is
	the parent of the newly created object within the GUI and a pointer to a QListViewItem which
	is the parent ot the newly created object within the aKpi browser.
	The QHBox hbox is the widget where the node hast to place all of it's GUI elements onto
	(if there are none just leave it empty).
*/
AcpiBase::AcpiBase(AcpiList* list, QWidget* wparent, QListViewItem* lvparent) : QHBox(wparent), QListViewItem(lvparent)
{
	name = "AcpiBase";
	this->list = list;
	this->wparent = wparent;
	pollType = none;
	hbox = static_cast<QHBox*>(this);
	hbox->hide();
}


// ------------------------------------------------------------
// AcpiBase::~AcpiBase
// ------------------------------------------------------------
/*!
	The destructor cleans up all items that have been created dynamically.
*/
AcpiBase::~AcpiBase()
{
	#ifdef ACPI_DEBUG
		cout << "Destructor AcpiBase (name: " << name << " path: " << path << ")" << endl;
	#endif

	lvitems.setAutoDelete(true);
	lvitems.clear();

  files.setAutoDelete(true); 	
	files.clear();

	children.setAutoDelete(true);
	children.clear();

	#ifdef ACPI_DEBUG
		cout << "done (" << name << ")" <<  endl;
	#endif
}


// ------------------------------------------------------------
// AcpiBase::indent
// This method returns QString value of given length.
// ------------------------------------------------------------
QString AcpiBase::indent(int indent)
{
	indent *= INDENT_MULT;
	QString sBuffer = "";
	for (int i = 0; i < indent; i++) {
		sBuffer.append(" ");
	}
	return sBuffer;
}


// ------------------------------------------------------------
// AcpiBase::parseDir
// ------------------------------------------------------------
/*!
	This method parses the given directory (basepath + "/" + path).
	To handle the files in this directory parseFiles() is called.
	For every subfolder that is found an object is created and added to the
	current object's children list. If the subfolder is of some special (known)
	type a special object is created (like AcpiBattery). For all other
	subfolders a generic AcpiNode is created.
*/
void AcpiBase::parseDir(QString basepath, QString path)
{
	if (basepath != "")
		this->path = basepath + "/" + path;
	else
		this->path = path;

	setText(0, path);  // set's the text of the QListViewItem

	parseFiles(this->path);  // handle all files that are in this folder

	QDir dir = QDir(this->path);

	dir.setFilter(QDir::Dirs | QDir::Hidden | QDir::NoSymLinks);
	dir.setSorting(QDir::Name);

	const QFileInfoList* list = dir.entryInfoList();
	QFileInfoListIterator it(*list);
	QFileInfo* fi;

	AcpiBase* node;

	while ((fi = it.current())) {

		if (fi->fileName() != ".." && fi->fileName() != ".") {

			// create objects for every subfolder that's in this
			// folder and append it to the children list

			if (path.contains("battery")) {
				node = new AcpiBattery(this->list, wparent, dynamic_cast<QListViewItem*>(this));

			} else if (path.contains("ac_adapter")) {
				node = new AcpiACAdapter(this->list, wparent, dynamic_cast<QListViewItem*>(this));

			} else if (path.contains("processor")) {
				node = new AcpiProcessor(this->list, wparent, dynamic_cast<QListViewItem*>(this));

			} else if (path.contains("thermal_zone")) {
				node = new AcpiThermal(this->list, wparent, dynamic_cast<QListViewItem*>(this));

			} else if (path.contains("toshiba")) {
				node = new AcpiToshiba(this->list, wparent, dynamic_cast<QListViewItem*>(this), this->path + "/" + fi->fileName());
				#ifdef ACPI_DEBUG
					cout << "Toshiba object created.\n";
				#endif

			}	else {
				node = new AcpiNode(this->list, wparent, dynamic_cast<QListViewItem*>(this));
			}

			node->parseDir(this->path, fi->fileName());
			children.append(node);

		}

		++it;
	}
}


// ------------------------------------------------------------
// AcpiBase::parseFiles
// ------------------------------------------------------------
/*!
	This method walks trough all the files in the given
	folder and calls parseDate for every single file.
*/
void AcpiBase::parseFiles(QString path)
{
	QDir dir = QDir(path);

	dir.setFilter(QDir::Files | QDir::Hidden | QDir::NoSymLinks);
	dir.setSorting(QDir::Name);

	const QFileInfoList* list = dir.entryInfoList();
	QFileInfoListIterator it(*list);
	QFileInfo* fi;

	while ((fi = it.current())) {
		parseData(path, fi->fileName());
		++it;
	}
}


// ------------------------------------------------------------
// AcpiBase::parseData
// ------------------------------------------------------------
/*!
	This method opens the given file and reads the data lines into
	an internal data-structure.
*/
void AcpiBase::parseData(QString path, QString filename)
{
	QMap<int, QString> content;
	content = loadFile(path + "/" + filename);

	QListViewItem* item = new QListViewItem(this, filename);
	lvitems.append(item);

	typFileItem* fitem = new typFileItem();

	int separator;
	bool polling;
	QString linebuffer, key, val;

	QMap<int, QString>::Iterator it;
	for (it = content.begin(); it != content.end(); ++it) {

		QString linebuffer = it.data();
		separator = linebuffer.find(QString(":"), 0, false);

		if (separator != -1) {
			key = (linebuffer.left(separator)).stripWhiteSpace();
			val = (linebuffer.mid(++separator)).stripWhiteSpace();
			polling = true;  // defualt polling policy

			fitem->insert(key, new AcpiData(key, val, item, it.key(), polling));
		} else {

			fitem->insert("[nokey]", new AcpiData("[nokey]", linebuffer, item, it.key(), false));
		}
	}

	files.insert(filename, fitem);  // insert newly parsed file into files list of this node
}


// ------------------------------------------------------------
// AcpiBase::print
// ------------------------------------------------------------
/*!
	This method prints the whole internal datastructure to the screen.
	It's used mostly for debuging and analysis.
*/
void AcpiBase::print(int ind)
{
	cout << indent(ind - INDENT_ADD * 2) << "+ " << path << "  (class: " << name<< ")" << endl;  // path

	QDictIterator<typFileItem> itFileItem(files);

	while (itFileItem.current()) {

		cout << indent(ind - INDENT_ADD) << "- " << itFileItem.currentKey() << endl;

		QDictIterator<AcpiData> itAcpiData(*itFileItem.current());

		while (itAcpiData.current()) {
			cout << indent(ind) << itAcpiData.currentKey() << ": ";
			cout << itAcpiData.current()->getValue();
			cout << "   (polling: "  << itAcpiData.current()->getPolling() << ")";
			cout << endl;
			++itAcpiData;
		}

		++itFileItem;
	}

	AcpiBase* node;
	for (node = children.first(); node != 0; node = children.next()) {
  	node->print(ind + INDENT_ADD);
	}
}



// ------------------------------------------------------------
// AcpiBase::poll
// ------------------------------------------------------------
/*!
	This method re-reads parts of the parsed data (only those items
	should be re-read that might change; in other words those items
	with polling == true)
	In addition to that polling can be limited to a certain item type
	(lets say batteries) by setting the type variable.
*/
void AcpiBase::poll(pollTypes type)
{
	QString linebuffer, key, val;
	int separator;
	bool mismatch = false;
	QDictIterator<typFileItem> itFiles(files);

	if (type == all || type == pollType)

	while (itFiles.current()) {
		QMap<int, QString> content;
		content = loadFile(path + "/" + itFiles.currentKey());

		QDictIterator<AcpiData> itAcpiData(*itFiles.current());

		while (itAcpiData.current()) {
			if (itAcpiData.current()->getPolling() && itAcpiData.current()->getValue() != NULL) {
				linebuffer = content[itAcpiData.current()->getPosition()];
#ifdef ACPI_DEBUG
				cout << "polling file: " << path + "/" + itFiles.currentKey() << " line: " << linebuffer << endl;
#endif
				separator = linebuffer.find(QString(":"), 0, false);

				if (separator != -1) {
					key = (linebuffer.left(separator)).stripWhiteSpace();
					val = (linebuffer.mid(++separator)).stripWhiteSpace();

					// check if some parts (battery or things like that) have been removed
					if (key.lower() == "present") {
						mismatch = itAcpiData.current()->isMismatch(key, val);
						if (mismatch) break;
					}

  				itAcpiData.current()->update(key, val);
				}
			}

			++itAcpiData;
		}

	// if we've found a 'present mismatch' we've to re-read the whole file instead of just
	// re-reading certain lines; to do so we remove the file from the internal data structure
	// and call parseData for this file which will re-add it to the internal data structure
	if (mismatch) {
#ifdef ACPI_DEBUG
			cout << "present mismatch detected - reparseing file: " << path << "/" << itFiles.currentKey() << endl;
#endif

		QListViewItem* lvp = itAcpiData.current()->getLVParent();
		QString file = itFiles.currentKey();
		(*itFiles.current()).setAutoDelete(true);
		files.setAutoDelete(true);
		files.remove(file);

		parseData(path, file);
		lvitems.remove(lvp);
		delete lvp;
		mismatch = false;
	}

		++itFiles;
	}


	AcpiBase* node;
	for (node = children.first(); node != 0; node = children.next()) {
  	node->poll(type);
	}
}


// ------------------------------------------------------------
// AcpiBase::emitSignals
// ------------------------------------------------------------
void AcpiBase::emitSignals()
{
	AcpiBase* node;
	for (node = children.first(); node != 0; node = children.next()) {
  	node->emitSignals();
	}
}


// ------------------------------------------------------------
// AcpiBase::updateGUI
// ------------------------------------------------------------
void AcpiBase::updateGUI()
{
	AcpiBase* node;
	for (node = children.first(); node != 0; node = children.next()) {
  	node->updateGUI();
	}
}


// ------------------------------------------------------------
// AcpiBase::seekLine
// ------------------------------------------------------------
/*!
	This method goes down to a certain line within the given file.
	This actually is some type of bad hack. Orignally I planed to use
	QFile's .at() methods. That doesn't work very good because if
	values change in /proc/acpi these ofsets get messed up. The only
	thing we can take for sure is the line number that will remain the
	same.
*/
void AcpiBase::seekLine(int line, QFile &file, QTextStream &stream)
{
	int count = 1;
	file.at(0);
	while (count < line) {
		++count;
		stream.readLine();
	}
}


/*!
	This method reads the content of the given file (full path) and returns it as
	QMap<int, QString> which maps line number to strings. Line numbering always
	starts with 1.
*/
QMap<int, QString> AcpiBase::loadFile(QString fileName)
{
	int count = 1;
	QMap<int, QString> content;

	QFile file(fileName);
	QTextStream filestream(&file);

	if (file.open(IO_ReadOnly)) {
		while (!filestream.eof()) {
			content[count] = filestream.readLine();
			++count;
		}
		file.close();
	}

	return content;
}

